@@ -23,6 +23,7 @@
 
 #include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
+#include "ephy-output-encoding.h"
 #include "ephy-web-view.h"
 
 #include <gio/gio.h>
@@ -109,7 +110,9 @@ web_resource_data_cb (WebKitWebResource     *resource,
                       EphyViewSourceRequest *request)
 {
   g_autofree guchar *data = NULL;
-  g_autofree char *escaped_str = NULL;
+  g_autofree char *data_str = NULL;
+  g_autofree char *encoded_str = NULL;
+  g_autofree char *encoded_uri = NULL;
   g_autoptr (GError) error = NULL;
   g_autofree char *html = NULL;
   gsize length;
@@ -120,8 +123,13 @@ web_resource_data_cb (WebKitWebResource     *resource,
     return;
   }
 
-  /* Warning: data is not a string, so we pass length here because it's not NUL-terminated. */
-  escaped_str = g_markup_escape_text ((const char *)data, length);
+  /* Convert data to a string */
+  data_str = g_malloc (length + 1);
+  memcpy (data_str, data, length);
+  data_str[length] = '\0';
+
+  encoded_str = ephy_encode_for_html_entity (data_str);
+  encoded_uri = ephy_encode_for_html_entity (webkit_web_resource_get_uri (resource));
 
   html = g_strdup_printf ("<head>"
                           "  <link rel='stylesheet' href='ephy-resource:///org/gnome/epiphany/highlightjs/nnfx-light.css' media='(prefers-color-scheme: no-preference), (prefers-color-scheme: light)'>"
@@ -136,8 +144,8 @@ web_resource_data_cb (WebKitWebResource     *resource,
                           "          hljs.initLineNumbersOnLoad();</script>"
                           "  <pre><code class='html'>%s</code></pre>"
                           "</body>",
-                          webkit_web_resource_get_uri (resource),
-                          escaped_str);
+                          encoded_uri,
+                          encoded_str);
 
   finish_uri_scheme_request (request, g_steal_pointer (&html), NULL);
 }
@@ -174,7 +182,9 @@ ephy_view_source_request_begin_get_source_from_uri (EphyViewSourceRequest *reque
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   WebKitWebContext *context = ephy_embed_shell_get_web_context (shell);
 
-  request->web_view = WEBKIT_WEB_VIEW (g_object_ref_sink (webkit_web_view_new_with_context (context)));
+  request->web_view = WEBKIT_WEB_VIEW (g_object_ref_sink (g_object_new (WEBKIT_TYPE_WEB_VIEW,
+                                                                        "web-context", context,
+                                                                        NULL)));
 
   g_assert (request->load_changed_id == 0);
   request->load_changed_id = g_signal_connect (request->web_view, "load-changed",
@@ -186,12 +196,9 @@ ephy_view_source_request_begin_get_source_from_uri (EphyViewSourceRequest *reque
 
 static gint
 embed_is_displaying_matching_uri (EphyEmbed *embed,
-                                  GUri      *uri)
+                                  char      *uri)
 {
   EphyWebView *web_view;
-  g_autoptr (GUri) view_uri = NULL;
-  g_autofree char *modified_uri = NULL;
-  g_autofree char *uri_string = NULL;
 
   if (ephy_embed_has_load_pending (embed))
     return -1;
@@ -200,18 +207,11 @@ embed_is_displaying_matching_uri (EphyEmbed *embed,
   if (ephy_web_view_is_loading (web_view))
     return -1;
 
-  view_uri = g_uri_parse (ephy_web_view_get_address (web_view),
-                          G_URI_FLAGS_NONE, NULL);
-  if (!view_uri)
-    return -1;
-
-  modified_uri = g_uri_to_string_partial (view_uri, G_URI_HIDE_FRAGMENT);
-  uri_string = g_uri_to_string (uri);
-  return strcmp (modified_uri, uri_string);
+  return g_strcmp0 (ephy_web_view_get_address (web_view), uri);
 }
 
 static WebKitWebView *
-get_web_view_matching_uri (GUri *uri)
+get_web_view_matching_uri (const char *uri)
 {
   EphyEmbedShell *shell;
   GtkWindow *window;
@@ -240,8 +240,7 @@ out:
 static void
 ephy_view_source_request_start (EphyViewSourceRequest *request)
 {
-  g_autoptr (GUri) uri = NULL;
-  g_autoptr (GUri) converted_uri = NULL;
+  const char *converted_uri;
   const char *original_uri;
   WebKitWebView *web_view;
 
@@ -249,38 +248,15 @@ ephy_view_source_request_start (EphyViewSourceRequest *request)
     g_list_prepend (request->source_handler->outstanding_requests, request);
 
   original_uri = webkit_uri_scheme_request_get_uri (request->scheme_request);
-  uri = g_uri_parse (original_uri, G_URI_FLAGS_ENCODED | G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
 
-  if (!uri || !g_uri_get_fragment (uri)) {
-    /* Can't assert because user could theoretically input something weird */
-    GError *error = g_error_new (WEBKIT_NETWORK_ERROR,
-                                 WEBKIT_NETWORK_ERROR_FAILED,
-                                 _("%s is not a valid URI"),
-                                 original_uri);
-    finish_uri_scheme_request (request, NULL, error);
-    g_error_free (error);
-    return;
-  }
-
-  /* Convert e.g. ephy-source://gnome.org#https to https://gnome.org */
-  converted_uri = g_uri_build (g_uri_get_flags (uri),
-                               g_uri_get_fragment (uri),
-                               g_uri_get_userinfo (uri),
-                               g_uri_get_host (uri),
-                               g_uri_get_port (uri),
-                               g_uri_get_path (uri),
-                               g_uri_get_query (uri),
-                               NULL);
-  g_assert (converted_uri);
+  /* Convert e.g. view-source:https://gnome.org to https://gnome.org */
+  converted_uri = original_uri + strlen (EPHY_VIEW_SOURCE_SCHEME) + 1;
 
   web_view = get_web_view_matching_uri (converted_uri);
   if (web_view)
     ephy_view_source_request_begin_get_source_from_web_view (request, WEBKIT_WEB_VIEW (web_view));
   else {
-    g_autofree char *modified_uri = NULL;
-
-    modified_uri = g_uri_to_string (converted_uri);
-    ephy_view_source_request_begin_get_source_from_uri (request, modified_uri);
+    ephy_view_source_request_begin_get_source_from_uri (request, converted_uri);
   }
 }
 
