@@ -18,22 +18,23 @@
  *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
 #include "ephy-data-view.h"
 
+#include <adwaita.h>
 #include <ctype.h>
 
 typedef struct {
   GtkWidget *box;
   GtkWidget *child;
-  GtkWidget *header_bar;
+  GtkWidget *window_title;
   GtkWidget *back_button;
-  GtkWidget *clear_all_button;
+  GtkWidget *clear_button;
   GtkWidget *search_bar;
   GtkWidget *search_entry;
   GtkWidget *search_button;
   GtkWidget *stack;
-  GtkWidget *empty_title_label;
-  GtkWidget *empty_description_label;
+  GtkWidget *empty_page;
   GtkWidget *spinner;
 
   gboolean is_loading : 1;
@@ -41,16 +42,25 @@ typedef struct {
   gboolean has_search_results : 1;
   gboolean can_clear : 1;
   char *search_text;
+  char *search_description;
 } EphyDataViewPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (EphyDataView, ephy_data_view, GTK_TYPE_BIN)
+static void ephy_data_view_buildable_init (GtkBuildableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (EphyDataView, ephy_data_view, GTK_TYPE_WIDGET,
+                         G_ADD_PRIVATE (EphyDataView)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
+                                                ephy_data_view_buildable_init))
+
+static GtkBuildableIface *parent_buildable_iface;
 
 enum {
   PROP_0,
   PROP_TITLE,
-  PROP_CLEAR_ALL_ACTION_NAME,
-  PROP_CLEAR_ALL_ACTION_TARGET,
-  PROP_CLEAR_ALL_DESCRIPTION,
+  PROP_CLEAR_ACTION_NAME,
+  PROP_CLEAR_ACTION_TARGET,
+  PROP_CLEAR_BUTTON_LABEL,
+  PROP_CLEAR_BUTTON_TOOLTIP,
   PROP_SEARCH_DESCRIPTION,
   PROP_EMPTY_TITLE,
   PROP_EMPTY_DESCRIPTION,
@@ -66,7 +76,7 @@ static GParamSpec *obj_properties[LAST_PROP];
 
 enum {
   BACK_BUTTON_CLICKED,
-  CLEAR_ALL_CLICKED,
+  CLEAR_BUTTON_CLICKED,
   LAST_SIGNAL,
 };
 
@@ -96,7 +106,7 @@ update (EphyDataView *self)
     gtk_spinner_stop (GTK_SPINNER (priv->spinner));
   }
 
-  gtk_widget_set_sensitive (priv->clear_all_button, has_data && priv->can_clear);
+  gtk_widget_set_sensitive (priv->clear_button, has_data && priv->can_clear);
   gtk_widget_set_sensitive (priv->search_button, has_data);
 }
 
@@ -108,9 +118,9 @@ on_back_button_clicked (GtkWidget    *button,
 }
 
 static void
-on_clear_all_button_clicked (EphyDataView *self)
+on_clear_button_clicked (EphyDataView *self)
 {
-  g_signal_emit (self, signals[CLEAR_ALL_CLICKED], 0);
+  g_signal_emit (self, signals[CLEAR_BUTTON_CLICKED], 0);
 }
 
 static void
@@ -120,55 +130,13 @@ on_search_entry_changed (GtkSearchEntry *entry,
   EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
   const char *text;
 
-  text = gtk_entry_get_text (GTK_ENTRY (entry));
+  text = gtk_editable_get_text (GTK_EDITABLE (entry));
   g_free (priv->search_text);
   priv->search_text = g_strdup (text);
 
   g_object_notify_by_pspec (G_OBJECT (self), obj_properties[PROP_SEARCH_TEXT]);
 }
 
-gboolean
-ephy_data_view_handle_event (EphyDataView *self,
-                             GdkEvent     *event)
-{
-  EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
-  GdkEventKey *key = (GdkEventKey *)event;
-  gint result;
-
-  /* Firstly, we check if the HdySearchBar can handle the event */
-  result = hdy_search_bar_handle_event (HDY_SEARCH_BAR (priv->search_bar), event);
-
-  if (result == GDK_EVENT_STOP)
-    return result;
-
-  /* Secondly, we check for the shortcuts implemented by the data views */
-  /* Ctrl + F */
-  if ((key->state & GDK_CONTROL_MASK) && key->keyval == GDK_KEY_f) {
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->search_button), TRUE);
-    return GDK_EVENT_STOP;
-  }
-
-  /* Shift + Delete */
-  if ((key->state & GDK_SHIFT_MASK) && key->keyval == GDK_KEY_Delete) {
-    gtk_button_clicked (GTK_BUTTON (priv->clear_all_button));
-    return GDK_EVENT_STOP;
-  }
-
-  /* Finally, we check for the Escape key */
-  if (key->keyval == GDK_KEY_Escape) {
-    if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->search_button)))
-      /* If the user presses the Escape key and the search bar is hidden,
-       * we want the event to have the same effect as clicking the back button*/
-      gtk_button_clicked (GTK_BUTTON (priv->back_button));
-    else
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->search_button), FALSE);
-
-    return GDK_EVENT_STOP;
-  }
-
-  return GDK_EVENT_PROPAGATE;
-}
-
 static void
 ephy_data_view_set_property (GObject      *object,
                              guint         prop_id,
@@ -180,26 +148,33 @@ ephy_data_view_set_property (GObject      *object,
 
   switch (prop_id) {
     case PROP_TITLE:
-      gtk_header_bar_set_title (GTK_HEADER_BAR (priv->header_bar), g_value_get_string (value));
+      adw_window_title_set_title (ADW_WINDOW_TITLE (priv->window_title), g_value_get_string (value));
       break;
-    case PROP_CLEAR_ALL_ACTION_NAME:
-      gtk_actionable_set_action_name (GTK_ACTIONABLE (priv->clear_all_button), g_value_get_string (value));
+    case PROP_CLEAR_ACTION_NAME:
+      gtk_actionable_set_action_name (GTK_ACTIONABLE (priv->clear_button), g_value_get_string (value));
       break;
-    case PROP_CLEAR_ALL_ACTION_TARGET:
-      gtk_actionable_set_action_target_value (GTK_ACTIONABLE (priv->clear_all_button), g_value_get_variant (value));
+    case PROP_CLEAR_ACTION_TARGET:
+      gtk_actionable_set_action_target_value (GTK_ACTIONABLE (priv->clear_button), g_value_get_variant (value));
       break;
-    case PROP_CLEAR_ALL_DESCRIPTION:
-      ephy_data_view_set_clear_all_description (self, g_value_get_string (value));
+    case PROP_CLEAR_BUTTON_LABEL:
+      ephy_data_view_set_clear_button_label (self, g_value_get_string (value));
+      break;
+    case PROP_CLEAR_BUTTON_TOOLTIP:
+      ephy_data_view_set_clear_button_tooltip (self, g_value_get_string (value));
       break;
     case PROP_SEARCH_DESCRIPTION:
-      gtk_entry_set_placeholder_text (GTK_ENTRY (priv->search_entry), g_value_get_string (value));
-      atk_object_set_description (gtk_widget_get_accessible (GTK_WIDGET (self)), g_value_get_string (value));
+      g_clear_pointer (&priv->search_description, g_free);
+      priv->search_description = g_value_dup_string (value);
+      gtk_accessible_update_property (GTK_ACCESSIBLE (self),
+                                      GTK_ACCESSIBLE_PROPERTY_DESCRIPTION,
+                                      priv->search_description,
+                                      -1);
       break;
     case PROP_EMPTY_TITLE:
-      gtk_label_set_text (GTK_LABEL (priv->empty_title_label), g_value_get_string (value));
+      adw_status_page_set_title (ADW_STATUS_PAGE (priv->empty_page), g_value_get_string (value));
       break;
     case PROP_EMPTY_DESCRIPTION:
-      gtk_label_set_text (GTK_LABEL (priv->empty_description_label), g_value_get_string (value));
+      adw_status_page_set_description (ADW_STATUS_PAGE (priv->empty_page), g_value_get_string (value));
       break;
     case PROP_IS_LOADING:
       ephy_data_view_set_is_loading (self, g_value_get_boolean (value));
@@ -230,25 +205,28 @@ ephy_data_view_get_property (GObject    *object,
 
   switch (prop_id) {
     case PROP_TITLE:
-      g_value_set_string (value, gtk_header_bar_get_title (GTK_HEADER_BAR (priv->header_bar)));
+      g_value_set_string (value, adw_window_title_get_title (ADW_WINDOW_TITLE (priv->window_title)));
       break;
-    case PROP_CLEAR_ALL_ACTION_NAME:
-      g_value_set_string (value, gtk_actionable_get_action_name (GTK_ACTIONABLE (priv->clear_all_button)));
+    case PROP_CLEAR_ACTION_NAME:
+      g_value_set_string (value, gtk_actionable_get_action_name (GTK_ACTIONABLE (priv->clear_button)));
       break;
-    case PROP_CLEAR_ALL_ACTION_TARGET:
-      g_value_set_variant (value, gtk_actionable_get_action_target_value (GTK_ACTIONABLE (priv->clear_all_button)));
+    case PROP_CLEAR_ACTION_TARGET:
+      g_value_set_variant (value, gtk_actionable_get_action_target_value (GTK_ACTIONABLE (priv->clear_button)));
       break;
-    case PROP_CLEAR_ALL_DESCRIPTION:
-      g_value_set_string (value, ephy_data_view_get_clear_all_description (self));
+    case PROP_CLEAR_BUTTON_LABEL:
+      g_value_set_string (value, ephy_data_view_get_clear_button_label (self));
+      break;
+    case PROP_CLEAR_BUTTON_TOOLTIP:
+      g_value_set_string (value, ephy_data_view_get_clear_button_tooltip (self));
       break;
     case PROP_SEARCH_DESCRIPTION:
-      g_value_set_string (value, gtk_entry_get_placeholder_text (GTK_ENTRY (priv->search_entry)));
+      g_value_set_string (value, priv->search_description);
       break;
     case PROP_EMPTY_TITLE:
-      g_value_set_string (value, gtk_label_get_text (GTK_LABEL (priv->empty_title_label)));
+      g_value_set_string (value, adw_status_page_get_title (ADW_STATUS_PAGE (priv->empty_page)));
       break;
     case PROP_EMPTY_DESCRIPTION:
-      g_value_set_string (value, gtk_label_get_text (GTK_LABEL (priv->empty_description_label)));
+      g_value_set_string (value, adw_status_page_get_description (ADW_STATUS_PAGE (priv->empty_page)));
       break;
     case PROP_SEARCH_TEXT:
       g_value_set_string (value, ephy_data_view_get_search_text (self));
@@ -271,6 +249,17 @@ ephy_data_view_get_property (GObject    *object,
   }
 }
 
+static void
+ephy_data_view_dispose (GObject *object)
+{
+  EphyDataView *self = EPHY_DATA_VIEW (object);
+  EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
+
+  g_clear_pointer (&priv->box, gtk_widget_unparent);
+
+  G_OBJECT_CLASS (ephy_data_view_parent_class)->dispose (object);
+}
+
 static void
 ephy_data_view_finalize (GObject *object)
 {
@@ -278,28 +267,22 @@ ephy_data_view_finalize (GObject *object)
   EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
 
   g_free (priv->search_text);
+  g_free (priv->search_description);
 
   G_OBJECT_CLASS (ephy_data_view_parent_class)->finalize (object);
 }
 
-static void
-ephy_data_view_add (GtkContainer *container,
-                    GtkWidget    *child)
+static gboolean
+find_shortuct_cb (EphyDataView *self)
 {
-  EphyDataView *self = EPHY_DATA_VIEW (container);
   EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
+  gboolean search;
 
-  if (!priv->box) {
-    GTK_CONTAINER_CLASS (ephy_data_view_parent_class)->add (container, child);
-    return;
-  }
+  search = gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (priv->search_bar));
 
-  g_assert (!priv->child);
+  gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (priv->search_bar), !search);
 
-  priv->child = child;
-  gtk_container_add (GTK_CONTAINER (priv->stack), child);
-
-  update (self);
+  return GDK_EVENT_STOP;
 }
 
 static void
@@ -307,95 +290,87 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
-  GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
 
   object_class->set_property = ephy_data_view_set_property;
   object_class->get_property = ephy_data_view_get_property;
+  object_class->dispose = ephy_data_view_dispose;
   object_class->finalize = ephy_data_view_finalize;
 
-  container_class->add = ephy_data_view_add;
-
   obj_properties[PROP_TITLE] =
     g_param_spec_string ("title",
-                         "Title",
-                         "The title used for the top HdyHeaderBar",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
-  obj_properties[PROP_CLEAR_ALL_ACTION_NAME] =
-    g_param_spec_string ("clear-all-action-name",
-                         "'Clear all' action name",
-                         "The name of the action associated to the 'Clear all' button",
+  obj_properties[PROP_CLEAR_ACTION_NAME] =
+    g_param_spec_string ("clear-action-name",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
-  obj_properties[PROP_CLEAR_ALL_ACTION_TARGET] =
-    g_param_spec_variant ("clear-all-action-target",
-                          "'Clear all' action target value",
-                          "The parameter for 'Clear all' action invocations",
+  obj_properties[PROP_CLEAR_ACTION_TARGET] =
+    g_param_spec_variant ("clear-action-target",
+                          NULL, NULL,
                           G_VARIANT_TYPE_ANY, NULL,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
-  obj_properties[PROP_CLEAR_ALL_DESCRIPTION] =
-    g_param_spec_string ("clear-all-description",
-                         "'Clear all' description",
-                         "The description of the 'Clear all' action",
+  obj_properties[PROP_CLEAR_BUTTON_LABEL] =
+    g_param_spec_string ("clear-button-label",
+                         NULL, NULL,
+                         NULL,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  obj_properties[PROP_CLEAR_BUTTON_TOOLTIP] =
+    g_param_spec_string ("clear-button-tooltip",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_SEARCH_DESCRIPTION] =
     g_param_spec_string ("search-description",
-                         "'Search' description",
-                         "The description of the 'Search' action",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_EMPTY_TITLE] =
     g_param_spec_string ("empty-title",
-                         "'Empty' title",
-                         "The title of the 'Empty' state page",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_EMPTY_DESCRIPTION] =
     g_param_spec_string ("empty-description",
-                         "'Empty' description",
-                         "The description of the 'Empty' state page",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_SEARCH_TEXT] =
     g_param_spec_string ("search-text",
-                         "Search text",
-                         "The text of the search entry",
+                         NULL, NULL,
                          NULL,
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_IS_LOADING] =
     g_param_spec_boolean ("is-loading",
-                          "Is loading",
-                          "Whether the dialog is loading its data",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_HAS_DATA] =
     g_param_spec_boolean ("has-data",
-                          "Has data",
-                          "Whether the dialog has data",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_HAS_SEARCH_RESULTS] =
     g_param_spec_boolean ("has-search-results",
-                          "Has search results",
-                          "Whether the dialog has search results",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_CAN_CLEAR] =
     g_param_spec_boolean ("can-clear",
-                          "Can clear",
-                          "Whether the data can be cleared",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
@@ -415,8 +390,8 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
    * Emitted when the user changes the contents of the internal #GtkEntry
    *
    */
-  signals[CLEAR_ALL_CLICKED] =
-    g_signal_new ("clear-all-clicked",
+  signals[CLEAR_BUTTON_CLICKED] =
+    g_signal_new ("clear-button-clicked",
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
                   0, NULL, NULL, NULL,
@@ -428,11 +403,10 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
                                                "/org/gnome/epiphany/gtk/data-view.ui");
 
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, box);
-  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, header_bar);
+  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, window_title);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, back_button);
-  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, clear_all_button);
-  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, empty_title_label);
-  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, empty_description_label);
+  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, clear_button);
+  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, empty_page);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, search_bar);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, search_button);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, search_entry);
@@ -440,8 +414,15 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, stack);
 
   gtk_widget_class_bind_template_callback (widget_class, on_back_button_clicked);
-  gtk_widget_class_bind_template_callback (widget_class, on_clear_all_button_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, on_clear_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_search_entry_changed);
+
+  gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
+
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_F, GDK_CONTROL_MASK,
+                                (GtkShortcutFunc)find_shortuct_cb, NULL);
+  gtk_widget_class_add_binding_signal (widget_class, GDK_KEY_Delete, GDK_SHIFT_MASK,
+                                       "clear-button-clicked", NULL);
 }
 
 static void
@@ -451,13 +432,46 @@ ephy_data_view_init (EphyDataView *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  hdy_search_bar_connect_entry (HDY_SEARCH_BAR (priv->search_bar), GTK_ENTRY (priv->search_entry));
+  gtk_search_bar_connect_entry (GTK_SEARCH_BAR (priv->search_bar),
+                                GTK_EDITABLE (priv->search_entry));
+
+  adw_status_page_set_icon_name (ADW_STATUS_PAGE (priv->empty_page),
+                                 APPLICATION_ID "-symbolic");
 
   update (self);
 }
 
+static void
+ephy_data_view_add_child (GtkBuildable *buildable,
+                          GtkBuilder   *builder,
+                          GObject      *child,
+                          const char   *type)
+{
+  EphyDataView *self = EPHY_DATA_VIEW (buildable);
+  EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
+
+  if (!priv->box || !GTK_IS_WIDGET (child)) {
+    parent_buildable_iface->add_child (buildable, builder, child, type);
+    return;
+  }
+
+  g_assert (!priv->child);
+
+  priv->child = GTK_WIDGET (child);
+  gtk_stack_add_child (GTK_STACK (priv->stack), priv->child);
+
+  update (self);
+}
+
+static void
+ephy_data_view_buildable_init (GtkBuildableIface *iface)
+{
+  parent_buildable_iface = g_type_interface_peek_parent (iface);
+  iface->add_child = ephy_data_view_add_child;
+}
+
 const gchar *
-ephy_data_view_get_clear_all_description (EphyDataView *self)
+ephy_data_view_get_clear_button_label (EphyDataView *self)
 {
   EphyDataViewPrivate *priv;
 
@@ -465,12 +479,12 @@ ephy_data_view_get_clear_all_description (EphyDataView *self)
 
   priv = ephy_data_view_get_instance_private (self);
 
-  return gtk_widget_get_tooltip_text (GTK_WIDGET (priv->clear_all_button));
+  return gtk_button_get_label (GTK_BUTTON (priv->clear_button));
 }
 
 void
-ephy_data_view_set_clear_all_description (EphyDataView *self,
-                                          const gchar  *description)
+ephy_data_view_set_clear_button_label (EphyDataView *self,
+                                       const gchar  *label)
 {
   EphyDataViewPrivate *priv;
 
@@ -478,12 +492,42 @@ ephy_data_view_set_clear_all_description (EphyDataView *self,
 
   priv = ephy_data_view_get_instance_private (self);
 
-  if (g_strcmp0 (gtk_widget_get_tooltip_text (GTK_WIDGET (priv->clear_all_button)), description) == 0)
+  if (g_strcmp0 (gtk_button_get_label (GTK_BUTTON (priv->clear_button)), label) == 0)
     return;
 
-  gtk_widget_set_tooltip_text (GTK_WIDGET (priv->clear_all_button), description);
+  gtk_button_set_label (GTK_BUTTON (priv->clear_button), label);
 
-  g_object_notify_by_pspec (G_OBJECT (self), obj_properties[PROP_CLEAR_ALL_DESCRIPTION]);
+  g_object_notify_by_pspec (G_OBJECT (self), obj_properties[PROP_CLEAR_BUTTON_LABEL]);
+}
+
+const gchar *
+ephy_data_view_get_clear_button_tooltip (EphyDataView *self)
+{
+  EphyDataViewPrivate *priv;
+
+  g_assert (EPHY_IS_DATA_VIEW (self));
+
+  priv = ephy_data_view_get_instance_private (self);
+
+  return gtk_widget_get_tooltip_text (GTK_WIDGET (priv->clear_button));
+}
+
+void
+ephy_data_view_set_clear_button_tooltip (EphyDataView *self,
+                                         const gchar  *tooltip)
+{
+  EphyDataViewPrivate *priv;
+
+  g_assert (EPHY_IS_DATA_VIEW (self));
+
+  priv = ephy_data_view_get_instance_private (self);
+
+  if (g_strcmp0 (gtk_widget_get_tooltip_text (GTK_WIDGET (priv->clear_button)), tooltip) == 0)
+    return;
+
+  gtk_widget_set_tooltip_text (GTK_WIDGET (priv->clear_button), tooltip);
+
+  g_object_notify_by_pspec (G_OBJECT (self), obj_properties[PROP_CLEAR_BUTTON_TOOLTIP]);
 }
 
 gboolean
