@@ -1627,7 +1627,7 @@ static void
 set_image_from_favicon (EphyApplicationDialogData *data)
 {
   g_autoptr (GIcon) icon = NULL;
-  g_autoptr (GdkTexture) icon_texture = webkit_web_view_get_favicon (WEBKIT_WEB_VIEW (data->view));
+  GdkTexture *icon_texture = webkit_web_view_get_favicon (WEBKIT_WEB_VIEW (data->view));
 
   icon = ephy_favicon_get_from_texture_scaled (icon_texture, 0, 0);
 
@@ -1648,8 +1648,11 @@ set_app_icon_from_filename (EphyApplicationDialogData *data,
                             const char                *filename)
 {
   g_autoptr (GdkPixbuf) pixbuf = NULL;
+  g_autoptr (GError) error = NULL;
 
-  pixbuf = gdk_pixbuf_new_from_file_at_size (filename, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE, NULL);
+  pixbuf = gdk_pixbuf_new_from_file_at_size (filename, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE, &error);
+  if (pixbuf == NULL)
+    g_warning ("Failed to create pixbuf for %s: %s", filename, error->message);
 
   if (pixbuf != NULL) {
     data->framed_pixbuf = frame_pixbuf (G_ICON (pixbuf), &data->icon_rgba, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
@@ -1658,9 +1661,8 @@ set_app_icon_from_filename (EphyApplicationDialogData *data,
     create_install_dialog_when_ready (data);
   }
   if (data->icon_v == NULL) {
-    g_warning ("Failed to get icon for web app %s, giving up", data->display_address);
-    ephy_application_dialog_data_free (data);
-    return;
+    g_warning ("Failed to get icon for web app %s, falling back to favicon", data->display_address);
+    set_image_from_favicon (data);
   }
 }
 
@@ -1685,7 +1687,6 @@ static void
 download_icon_and_set_image (EphyApplicationDialogData *data)
 {
   g_autofree char *destination = NULL;
-  g_autofree char *tmp_filename = NULL;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
   data->download = webkit_network_session_download_uri (ephy_embed_shell_get_network_session (shell),
@@ -1698,8 +1699,7 @@ download_icon_and_set_image (EphyApplicationDialogData *data)
    * instead of using this workaround. */
   g_object_set_data (G_OBJECT (data->download), "ephy-download-set", GINT_TO_POINTER (TRUE));
 
-  tmp_filename = ephy_file_tmp_filename (".ephy-download-XXXXXX", NULL);
-  destination = g_build_filename (ephy_file_tmp_dir (), tmp_filename, NULL);
+  destination = g_build_filename (ephy_file_tmp_dir (), data->icon_href, NULL);
   webkit_download_set_destination (data->download, destination);
 
   g_signal_connect (data->download, "finished",
